==> spellathon-5.47.tar.gz: spellathon-5.47/src/logic/dispwordsbuf.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cstring>
#include <algorithm>
#include <cctype>
#include <cassert>

#include "dispwordsbuf.h"
#include "util.h"
#include "defines.h"

namespace Spellathon
{
	void 
	DispWordsBuf::init(const vector<string> &w, int wpl, int fw)
	{
		assert(w.size() > 0);

		words_.clear();
		setwords_.clear();
		words_.reserve(128);
		setwords_.reserve(128);
		fmt_.reserve(128*(32+fw_));

		hlevel_		= 0;
		words_		= w;
		wpl_		= wpl;
		fw_			= fw;
		solved_		= false;
		idx_		= -1;
		entered7_	= 0;

#ifdef WIN32
		ansifmt_[FMT_BEG] = ""; 
		ansifmt_[FMT_BEG2] = ""; 
		ansifmt_[FMT_BEG3] = ""; 
		ansifmt_[FMT_BEG4] = ""; 
		ansifmt_[FMT_BEG5] = ""; 
		ansifmt_[FMT_BEG6] = ""; 
		ansifmt_[FMT_BEG7] = ""; 
		ansifmt_[FMT_BEG8] = ""; 
		ansifmt_[FMT_END] = ""; 
		ansifmt_[FMT_END2] = ""; 
		ansifmt_[FMT_END3] = ""; 
		ansifmt_[FMT_END4] = ""; 
		ansifmt_[FMT_END5] = ""; 
		ansifmt_[FMT_END6] = ""; 
		ansifmt_[FMT_END7] = ""; 
		ansifmt_[FMT_END8] = ""; 
		ansifmt_[FMT_NL] = "\n"; 
		ansifmt_[FMT_SPC] = " "; 
#else
		ansifmt_[FMT_BEG] = "\033[1m"; 
		ansifmt_[FMT_BEG2] = "\033[1m"; 
		ansifmt_[FMT_BEG3] = "<\033[1m"; 
		ansifmt_[FMT_BEG4] = "<\033[1m"; 
		ansifmt_[FMT_BEG5] = ""; 
		ansifmt_[FMT_BEG6] = "<"; 
		ansifmt_[FMT_BEG7] = "<"; 
		ansifmt_[FMT_BEG8] = ""; 
		ansifmt_[FMT_END] = "\033[m"; 
		ansifmt_[FMT_END2] = "\033[m"; 
		ansifmt_[FMT_END3] = "\033[m>"; 
		ansifmt_[FMT_END4] = "\033[m>"; 
		ansifmt_[FMT_END5] = ""; 
		ansifmt_[FMT_END6] = ">"; 
		ansifmt_[FMT_END7] = ">"; 
		ansifmt_[FMT_END8] = ""; 
		ansifmt_[FMT_NL] = "\n"; 
		ansifmt_[FMT_SPC] = " "; 
#endif /* WIN32 */

		htmlfmt_[FMT_BEG] = "<i style=\"color: darkgreen;\"><b>"; 
		htmlfmt_[FMT_BEG2] = "<b style=\"color: darkgreen;\">"; 
		htmlfmt_[FMT_BEG3] = "<i style=\"color: #CC6633;\"><b>"; 
		htmlfmt_[FMT_BEG4] = "<b style=\"color: #CC6633;\">"; 
		htmlfmt_[FMT_BEG5] = "<font style=\"color: #606060;\">"; 
		htmlfmt_[FMT_BEG6] = "<font style=\"color: #003399;\">"; 
		htmlfmt_[FMT_BEG7] = "<font style=\"color: #000099;\">"; 
		htmlfmt_[FMT_BEG8] = "<font style=\"color: #101010;\">"; 
		htmlfmt_[FMT_END] = "</b></i>"; 
		htmlfmt_[FMT_END2] = "</b>"; 
		htmlfmt_[FMT_END3] = "</b></i>"; 
		htmlfmt_[FMT_END4] = "</b>"; 
		htmlfmt_[FMT_END5] = "</font>"; 
		htmlfmt_[FMT_END6] = "</font>"; 
		htmlfmt_[FMT_END7] = "</font>"; 
		htmlfmt_[FMT_END8] = "</font>"; 
		htmlfmt_[FMT_NL] = "<br>"; 
		htmlfmt_[FMT_SPC] = "&nbsp;"; 

		sz_ = (words_.size() * fw_) + 1;
	
		destroy();
		widx_ = (char (*)[2])memalloc(words_.size()*2);
		memset(widx_, 0, words_.size()*2);
		sol_ = (char *)memalloc(sz_ * sizeof(char));
		hint_ = (char *)memalloc(sz_ * sizeof(char));
	
		/* fill the solution buf with all the sol. words with required formatting */
		int i;
		for (i = 0; i < (int)words_.size(); i++) {
			sprintf(&sol_[i*fw_], "%-*s", fw_, words_[i].c_str());
			widx_[i][1] = strlen(words_[i].c_str());
		}
		sol_[(i*fw_)] = '\0';
	
		/* hint buf initially has all the words blanked out */
		for (int i = 0; i < sz_; i++) {
			if (islower(sol_[i])) 
				hint_[i] = '-';
			else 
				hint_[i] = sol_[i];
		}
	}
	
	int 
	DispWordsBuf::setWord(const string w)
	{
		p_ = std::find(setwords_.begin(), setwords_.end(), w);
		if (p_ != setwords_.end())
			/* already set */
			return 0;
	
		p_ = std::find(words_.begin(), words_.end(), w);
		if (p_ == words_.end())
			/* not found */
			return -1;
	
		/* found */
		idx_ = p_ - words_.begin();
		widx_[idx_][0] = 1;
		strncpy(&hint_[idx_*fw_], w.c_str(), strlen(w.c_str()));
		setwords_.push_back(w);
		if (widx_[idx_][1] == MAX_LETRS)
			entered7_++;
	
		return 1;
	}
	
	bool 
	DispWordsBuf::setHint()
	{
		if (hlevel_ >= 2)
			/* only 2 hints allowed */
			return false;
	
#ifdef WIN32
		srand(time(0));
#else
		srandom(time(0));
#endif /* WIN32 */
	
		for (int i = 0; i < (int)words_.size(); i++) {
			int c = words_[i].size();
			int j = c/3;	/* reveal 1/3'rd of the letters */
			do {
				int r = randnum(0, words_[i].size());
				if (!islower(hint_[(i*fw_) + r])) {
					hint_[(i*fw_) + r] = sol_[(i*fw_) + r];
					j--;
				}
			} while (c-- && j);
		}
	
		hlevel_++;
		return true;
	}

	void
	DispWordsBuf::destroy() 
	{ 
		free(sol_);		sol_ = 0;
		free(hint_);	hint_ = 0;
		free(widx_);	widx_ = 0;
	}

	const char *
	DispWordsBuf::format(const char *buf, const char *fmt_type[])
	{
		fmt_.clear();
		for (int i = 0; i < (int)words_.size(); i++) {
			if (widx_[i][0]) {
				int fb, fe;
				if (widx_[i][1] == 7) {
					if (solved_) {
						fb = FMT_BEG3;
						fe = FMT_END3;
					} else {
						if (idx_ == i) {
							fb = FMT_BEG3;
							fe = FMT_END3;
						} else {
							fb = FMT_BEG4;
							fe = FMT_END4;
						}
					}
				} else {
					if (solved_) {
						fb = FMT_BEG;
						fe = FMT_END;
					
					} else {
							if (idx_ == i) {
								fb = FMT_BEG;
								fe = FMT_END;
							} else {
								fb = FMT_BEG2;
								fe = FMT_END2;
							}
					}
				}
				if (idx_ == i)
					fmt_.append(">");
				fmt_.append(fmt_type[fb]);
				fmt_.append(&buf[i*fw_], widx_[i][1]);
				fmt_.append(fmt_type[fe]);
			} else {
				int fb, fe;
				if (widx_[i][1] == 7) {
					if (solved_) {
						fb = FMT_BEG7;
						fe = FMT_END7;
					} else {
						fb = FMT_BEG6;
						fe = FMT_END6;
					}
				} else {
					if (solved_) {
						fb = FMT_BEG8;
						fe = FMT_END8;
					
					} else {
						fb = FMT_BEG5;
						fe = FMT_END5;
					}
				}
				fmt_.append(fmt_type[fb]);
				fmt_.append(&buf[i*fw_], widx_[i][1]);
				fmt_.append(fmt_type[fe]);
			}
			for (int k = idx_==i ? fw_-widx_[i][1]-1 : fw_-widx_[i][1]; 
				 k > 0; --k) {
				fmt_.append(fmt_type[FMT_SPC]);
			}
			if ((i+1)%wpl_ == 0) {
					fmt_.append(fmt_type[FMT_NL]);
			}
		}
		if (words_.size() % wpl_ != 0)
			fmt_.append(fmt_type[FMT_NL]);

		return fmt_.c_str();
	}
}

==> spellathon-5.53.tar.gz: spellathon-5.53/src/logic/dispwordsbuf.cc <==
/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cstring>
#include <algorithm>
#include <cctype>
#include <cassert>

#include "dispwordsbuf.h"
#include "util.h"
#include "defines.h"

namespace Spellathon
{
	DispWordsBuf::DispWordsBuf() 
		: fw_(9), wpl_(4) 
	{
		sol_.reserve(128);
		hint_.reserve(128);
		setidx_.reserve(128);
		fmt_.reserve(128*(32+fw_));

#ifdef WIN32
		ansifmt_[FMT_BEG] = ""; 
		ansifmt_[FMT_BEG2] = ""; 
		ansifmt_[FMT_BEG3] = ""; 
		ansifmt_[FMT_BEG4] = ""; 
		ansifmt_[FMT_BEG5] = ""; 
		ansifmt_[FMT_BEG6] = ""; 
		ansifmt_[FMT_BEG7] = ""; 
		ansifmt_[FMT_BEG8] = ""; 
		ansifmt_[FMT_END] = ""; 
		ansifmt_[FMT_END2] = ""; 
		ansifmt_[FMT_END3] = ""; 
		ansifmt_[FMT_END4] = ""; 
		ansifmt_[FMT_END5] = ""; 
		ansifmt_[FMT_END6] = ""; 
		ansifmt_[FMT_END7] = ""; 
		ansifmt_[FMT_END8] = ""; 
		ansifmt_[FMT_NL] = "\n"; 
		ansifmt_[FMT_SPC] = " "; 
#else
		ansifmt_[FMT_BEG] = "\033[1m"; 
		ansifmt_[FMT_BEG2] = "\033[1m"; 
		ansifmt_[FMT_BEG3] = "<\033[1m"; 
		ansifmt_[FMT_BEG4] = "<\033[1m"; 
		ansifmt_[FMT_BEG5] = ""; 
		ansifmt_[FMT_BEG6] = "<"; 
		ansifmt_[FMT_BEG7] = "<"; 
		ansifmt_[FMT_BEG8] = ""; 
		ansifmt_[FMT_END] = "\033[m"; 
		ansifmt_[FMT_END2] = "\033[m"; 
		ansifmt_[FMT_END3] = "\033[m>"; 
		ansifmt_[FMT_END4] = "\033[m>"; 
		ansifmt_[FMT_END5] = ""; 
		ansifmt_[FMT_END6] = ">"; 
		ansifmt_[FMT_END7] = ">"; 
		ansifmt_[FMT_END8] = ""; 
		ansifmt_[FMT_NL] = "\n"; 
		ansifmt_[FMT_SPC] = " "; 
#endif /* WIN32 */

		htmlfmt_[FMT_BEG] = "<i style=\"color: green;\"><b>"; 
		htmlfmt_[FMT_BEG2] = "<b style=\"color: green;\">"; 
		htmlfmt_[FMT_BEG3] = "<i style=\"color: #CC6633;\"><b>"; 
		htmlfmt_[FMT_BEG4] = "<b style=\"color: #CC6633;\">"; 
		htmlfmt_[FMT_BEG5] = "<font style=\"color: #606060;\">"; 
		htmlfmt_[FMT_BEG6] = "<font style=\"color: #003399;\">"; 
		htmlfmt_[FMT_BEG7] = "<font style=\"color: #000099;\">"; 
		htmlfmt_[FMT_BEG8] = "<font style=\"color: #101010;\">"; 
		htmlfmt_[FMT_END] = "</b></i>"; 
		htmlfmt_[FMT_END2] = "</b>"; 
		htmlfmt_[FMT_END3] = "</b></i>"; 
		htmlfmt_[FMT_END4] = "</b>"; 
		htmlfmt_[FMT_END5] = "</font>"; 
		htmlfmt_[FMT_END6] = "</font>"; 
		htmlfmt_[FMT_END7] = "</font>"; 
		htmlfmt_[FMT_END8] = "</font>"; 
		htmlfmt_[FMT_NL] = "<br>"; 
		htmlfmt_[FMT_SPC] = "&nbsp;"; 
	}

	void 
	DispWordsBuf::init(const vector<string> &words, int wpl, int fw)
	{
		assert(w.size() > 0);

		sol_.clear();
		hint_.clear();
		setidx_.clear();

		hlevel_		= 0;
		wpl_		= wpl;
		fw_			= fw;
		solved_		= false;
		curidx_		= -1;
		entered7_	= false;
		nsetwords_	= 0;

		sol_ = words;

		for (int i = 0; i < (int)words.size(); i++) {
			hint_.push_back("~");	
			setidx_.push_back(0);
		}
	}
	
	int 
	DispWordsBuf::setWord(const string word)
	{
		p_ = std::find(sol_.begin(), sol_.end(), word);
		if (p_ == sol_.end())
			/* not found */
			return -1;
	
		/* found */
		curidx_ = p_ - sol_.begin();
		if (setidx_[curidx_])
			/* already entered */
			return 0;

		setidx_[curidx_] = true;
		hint_[curidx_] = word;
		nsetwords_++;

		if (sol_[curidx_].size() == MAX_LETRS)
			entered7_ = true;
		if (nsetwords_ == (int)sol_.size())
			setSolved();
	
		return 1;
	}
	
	bool 
	DispWordsBuf::setHint()
	{
		if (hlevel_ >= 2) {
			/* only 2 hints allowed */
			return false;
		} 

		if (hlevel_ == 0) {
			for (int i = 0; i < (int)sol_.size(); i++) {
				if (!setidx_[i])
					hint_[i] = string(sol_[i].size(), '-');
			}
		}
	
		for (int i = 0; i < (int)hint_.size(); i++) {
			int c = sol_[i].size();
			int j = c/3;	/* reveal 1/3'rd of the letters */
			do {
				int r = randnum(0, sol_[i].size());
				if (!islower(hint_[i][r])) {
					hint_[i][r] = sol_[i][r];
					j--;
				}
			} while (c-- && j);
		}
	
		hlevel_++;
		return true;
	}

	const char *
	DispWordsBuf::format(vector<string> &buf, const char *fmt_type[])
	{
		fmt_.clear();

		for (int i = 0; i < (int)buf.size(); i++) {
			if (setidx_[i]) {
				int fb, fe;
				if (buf[i].size() == 7) {
					if (solved_) {
						fb = FMT_BEG3;
						fe = FMT_END3;
					} else {
						if (curidx_ == i) {
							fb = FMT_BEG3;
							fe = FMT_END3;
						} else {
							fb = FMT_BEG4;
							fe = FMT_END4;
						}
					}
				} else {
					if (solved_) {
						fb = FMT_BEG;
						fe = FMT_END;
					
					} else {
							if (curidx_ == i) {
								fb = FMT_BEG;
								fe = FMT_END;
							} else {
								fb = FMT_BEG2;
								fe = FMT_END2;
							}
					}
				}
				if (curidx_ == i)
					fmt_.append(">");
				fmt_.append(fmt_type[fb]);
				fmt_ += buf[i];
				fmt_.append(fmt_type[fe]);
			} else {
				int fb, fe;
				if (buf[i].size() == 7) {
					if (solved_) {
						fb = FMT_BEG7;
						fe = FMT_END7;
					} else {
						fb = FMT_BEG6;
						fe = FMT_END6;
					}
				} else {
					if (solved_) {
						fb = FMT_BEG8;
						fe = FMT_END8;
					
					} else {
						fb = FMT_BEG5;
						fe = FMT_END5;
					}
				}
				fmt_.append(fmt_type[fb]);
				fmt_ += buf[i];
				fmt_.append(fmt_type[fe]);
			}
			for (int k = (curidx_==i) ? (fw_-buf[i].size()-1) : (fw_-buf[i].size())
				 ;k > 0; --k) {
				fmt_.append(fmt_type[FMT_SPC]);
			}
			if ((i+1)%wpl_ == 0) {
					fmt_.append(fmt_type[FMT_NL]);
			}
		}
		if (buf.size() % wpl_ != 0)
			fmt_.append(fmt_type[FMT_NL]);

		return fmt_.c_str();
	}
}
